#!/bin/bash

#adapted from http://www.zmanda.com/quick-backup-setup.html

#1. Проверка, установлены ли все компоненты AMANDA  
#2. Конфигурирование
#3. Бекап
#4. Восстановление

TEST="amanda"
RC=0
curdir=$(pwd)

#1:

dpkg -l amanda 2>&1 >/dev/null || RC=$?
echo TINFO "INIT: checking if package amanda is installed." | tee results_${TEST}
if [ $RC -ne 0 ]; then
	echo TBROK "INIT: package amanda is not installed. Exiting test." | tee -a results_${TEST}
	exit $RC
fi

echo TINFO "INIT: checking if amanda executables are present" | tee -a results_${TEST}
which amdump  >tst_${TEST}.err 2>&1 || RC=$?
if [ $RC -ne 0 ]
then
	echo TBROK "INIT: command amdump not found. Exiting test." | tee -a results_${TEST}
	exit $RC
fi

which amrestore  >tst_${TEST}.err 2>&1 || RC=$?
if [ $RC -ne 0 ]
then
	echo TBROK "INIT: command amrestore not found. Exiting test." | tee -a results_${TEST}
	exit $RC
fi

which amrmtape  >tst_${TEST}.err 2>&1 || RC=$?
if [ $RC -ne 0 ]
then
	echo TBROK "INIT: command amrmtape not found. Exiting test." | tee -a results_${TEST}
	exit $RC
fi

which amcheckdump  >tst_${TEST}.err 2>&1 || RC=$?
if [ $RC -ne 0 ]
then
	echo TBROK "INIT: command amcheckdump not found. Exiting test." | tee -a results_${TEST}
	exit $RC
fi

#Создание пользователя
echo TINFO "INIT: checking if amanda user has been created" | tee -a results_${TEST}
grep "^amanda:" /etc/passwd > /dev/null
if [ $? -ne 0 ]
then
	echo TINFO "INIT: user amanda not found, creating..." | tee -a results_${TEST}
	useradd -d /home/amanda -g amanda -G backup -U -m amanda 	
fi

#2:Конфигурирование

echo TINFO "INIT: setting up amanda for testing" | tee -a results_${TEST}
#in /etc/xinetd.d create ONE file, amandaserver
cp amandaserver /etc/xinetd.d/
#Добавление строки в конец конфигурационного файла 
[[ $(grep "includedir /etc/xinetd.d" /etc/xinetd.conf) ]] || echo -e "\nincludedir /etc/xinetd.d" >> /etc/xinetd.conf

#Перезагрузка xinetd
/etc/init.d/xinetd restart
sleep 7

#Проверка, слушает ли сервер
netstat -ln | grep 10080
if [ $? -ne 0 ]
then
	echo TBROK "INIT: amanda server is not listening" | tee -a results_${TEST}
	RC=1
	exit $RC
fi

#check/create & chown /home/amanda (or whichever home dir is in /etc/passwd)
amandauser=$(grep "^amanda" /etc/passwd | cut -d ':' -f 1)
amandahomedir=$(grep $amandauser /etc/passwd | cut -d ':' -f 6)
[[ -d $amandahomedir ]] || (mkdir $amandahomedir ; chown amanda.backup $amandahomedir)

#check/create & chown /media/BACKUP
tmp="/tmp/amanda/DailySet1"
if [[ -d $tmp ]] 
then
    echo "LOG: $tmp already exist" 
else    
    echo "LOG: Create $tmp"
    mkdir -p $tmp
    chown -R amanda.backup /tmp/amanda 
fi

media="/media/BACKUP/DailySet1"
if [[ -d $media ]] 
then
    echo "LOG: $media already exist" 
else    
    echo "LOG: Create $media"
    mkdir -p $media
    chown -R amanda.backup /media/BACKUP
fi

#/home/amanda/.amandahosts
cp amandahosts ${amandahomedir}/.amandahosts
#(make sure localhost entry is in /etc/hosts)
[[ $(grep localhost /etc/hosts) ]] || echo -e "\n127.0.0.1\tlocalhost" >> /etc/hosts

#check & mkdir /etc/amanda
etc="/etc/amanda/DailySet1"
if [[ -d $etc ]] 
then
    echo "LOG: $etc already exist"
else 
    echo "LOG: Create $etc"
    mkdir -p $etc && chown -R amanda.backup $etc
fi
cp amanda-client.conf /etc/amanda
cp amanda.conf $etc
cp disklist $etc

#mkdir -p /tmp/vtapes/DailySet1/slots
slotsdir="/tmp/amanda/vtapes/DailySet1/slots"
if [[ -d $slotsdir ]]
then   
    echo "LOG: $slotsdir already exist"
else
    echo "LOG: Create $slotsdir"
    (mkdir -p $slotsdir ; chown amanda.backup $slotsdir)
fi
for i in {1..4}; do sudo -u amanda mkdir $slotsdir/slot$i >/dev/null 2>&1; done
ln -s $slotsdir/slot1 $slotsdir/data

#mkdir -p /media/BACKUP/dumps/
dumpdir="/media/BACKUP/DailySet1/dumps/"
[[ -d $dumpdir ]] || (mkdir -p $dumpdir ; chown amanda.backup $dumpdir)

testdir="/tmp/tst"

#Работа с виртуальными лентами

[[ -d $slotsdir ]] && rm -rf $slotsdir/slot*/*
echo TINFO "INIT: initializing virtual tapes" | tee -a results_${TEST}
[[ -f /etc/amanda/DailySet1/tapelist ]] && rm /etc/amanda/DailySet1/tapelist
touch /etc/amanda/DailySet1/tapelist
for i in {1..3}; do sudo -u amanda amrmtape --quiet --erase --cleanup DailySet1 DailySet1-0${i} >/dev/null 2>&1; done
for i in {1..3}; do sudo -u amanda amrmtape --quiet --erase --cleanup DailySet1 DailySet1-0${i} >/dev/null 2>&1; done
echo TINFO "INIT: labeling vtapes" | tee -a results_${TEST}
for i in {1..3}; do sudo -u amanda amlabel DailySet1 DailySet1-0${i} ; done
sudo -u amanda amtape DailySet1 reset 
sudo -u amanda amtape DailySet1 clean 

#3.1:Осуществление бекапа

echo TINFO "Test #1: initiating backup" | tee -a results_${TEST}
sudo -u amanda amdump DailySet1


#Проверка успешности бекапа
cat $(ls -art /media/BACKUP/DailySet1/log.* | tail -1) | grep SUCCESS 

if [ $? -ne 0 ]
then
    cat /media/BACKUP/DailySet1/log.*
	echo TFAIL "Test #1: backup verification failed. No reason to test restore, exiting" | tee -a results_${TEST}
	RC=1
	exit $RC
else	
	echo TPASS "Test #1: backup successful" | tee -a results_${TEST}
fi

#3.2:Восстановление

echo TINFO "Test #2: initiating restore" | tee -a results_${TEST}
restoredir="/media/BACKUP/restore"

if [[ -d $restoredir ]] 
then
    echo "LOG: Restore directory $restoredir already exist"
else 
    echo "LOG: Create restore directory$restoredir"
    mkdir -p $restoredir
fi

#(echo sethost localhost; echo lcd $restoredir; echo addx .; echo extract; echo y; echo y) | amrecover -h localhost -C DailySet1 
(echo sethost localhost; echo setdisk ${testdir}; echo lcd $restoredir; echo cd ${testdir}; echo addx .; echo extract; echo y; echo y) | amrecover -h localhost -C DailySet1 >/dev/null 2>&1 

diff -r $testdir $restoredir >/dev/null 2>&1

if [ $? -eq 0 ]
then
	echo TPASS "Test #2: restore successful" | tee -a results_${TEST}
else
	echo TFAIL "Test #2: restore failed, directories are different" | tee -a results_${TEST}
	RC=1
fi

echo TINFO "Cleanup..." | tee -a results_${TEST}
#rm -rf $testdir
rm -rf /tmp/amanda
rm -rf /etc/amanda
rm -f /etc/xinetd.d/amandaserver
rm -rf /media/BACKUP/*

exit $RC



